@@ -358,15 +358,19 @@ static int dbbt_find_token(qdbint_t db, qdb_datum key, bt_record_t record,
 	}
 
 	while (*offset > 0) {
+		int len;
+
 		if (dbbt_read_record(db, *offset, record)) {
 			*offset = 0;
 			break;
 		}
 
-		x = strncmp(record->token, key.data, key.size);
-		if (strlen(record->token) < key.size) {
+		x = strncmp((char *) (record->token), (char *) (key.data),
+			    key.size);
+		len = strlen((char *) (record->token));
+		if (len < key.size) {
 			x = -1;
-		} else if (strlen(record->token) > key.size) {
+		} else if (len > key.size) {
 			x = 1;
 		}
 
@@ -461,7 +465,7 @@ int qdb_btree_identify(const char *file)
 
 	fclose(fptr);
 
-	if (strncmp(buf, "QSF1", 4) == 0)
+	if (strncmp((char *) buf, "QSF1", 4) == 0)
 		return 1;
 
 	return 0;
@@ -900,8 +904,8 @@ qdb_datum qdb_btree_firstkey(qdbint_t db)
 
 	record.token[BT_TOKEN_MAX - 1] = 0;
 
-	newkey.data = strdup(record.token);
-	newkey.size = strlen(record.token);
+	newkey.data = (unsigned char *) strdup((char *) (record.token));
+	newkey.size = strlen((char *) (record.token));
 
 	return newkey;
 }
@@ -943,8 +947,8 @@ qdb_datum qdb_btree_nextkey(qdbint_t db, qdb_datum key)
 
 	record.token[BT_TOKEN_MAX - 1] = 0;
 
-	newkey.data = strdup(record.token);
-	newkey.size = strlen(record.token);
+	newkey.data = (unsigned char *) strdup((char *) (record.token));
+	newkey.size = strlen((char *) (record.token));
 
 	return newkey;
 }
@@ -962,7 +966,7 @@ static int qdb_btree_delete__rewrite(qdbint_t db, bt_record_t record)
 	int x;
 
 	key.data = record->token;
-	key.size = strlen(record->token);
+	key.size = strlen((char *) (record->token));
 	val.data = (void *) (record->data);
 	val.size = 3 * sizeof(long);
 
