@@ -251,15 +251,19 @@ static int dbbt_find_token(qdbint_t db, qdb_datum key, bt_record_t record,
 	}
 
 	while (*offset > 0) {
+		int len;
+
 		if (dbbt_read_record(db, *offset, record)) {
 			*offset = 0;
 			break;
 		}
 
-		x = strncmp(record->token, key.data, key.size);
-		if (strlen(record->token) < key.size) {
+		x = strncmp((char *) (record->token), (char *) (key.data),
+			    key.size);
+		len = strlen((char *) (record->token));
+		if (len < key.size) {
 			x = -1;
-		} else if (strlen(record->token) > key.size) {
+		} else if (len > key.size) {
 			x = 1;
 		}
 
@@ -692,8 +696,8 @@ qdb_datum qdb_obtree_firstkey(qdbint_t db)
 
 	record.token[BT_TOKEN_MAX - 1] = 0;
 
-	key.data = strdup(record.token);
-	key.size = strlen(record.token);
+	key.data = (unsigned char *) strdup((char *) (record.token));
+	key.size = strlen((char *) (record.token));
 
 	return key;
 }
@@ -735,8 +739,8 @@ qdb_datum qdb_obtree_nextkey(qdbint_t db, qdb_datum key)
 
 	record.token[BT_TOKEN_MAX - 1] = 0;
 
-	newkey.data = strdup(record.token);
-	newkey.size = strlen(record.token);
+	newkey.data = (unsigned char *) strdup((char *) (record.token));
+	newkey.size = strlen((char *) (record.token));
 
 	return newkey;
 }
@@ -754,7 +758,7 @@ static int qdb_obtree_delete__reposition(qdbint_t db, unsigned long offset,
 	qdb_datum key;
 	int x;
 
-	key.data = token;
+	key.data = (unsigned char *) token;
 	key.size = strlen(token);
 
 	x = dbbt_find_token(db, key, &record, &offs, &parent);
@@ -838,10 +842,10 @@ int qdb_obtree_delete(qdbint_t db, qdb_datum key)
 	 */
 	if (record.lower > 0)
 		qdb_obtree_delete__reposition(db, record.lower,
-					      reclower.token);
+					      (char *) (reclower.token));
 	if (record.higher > 0)
 		qdb_obtree_delete__reposition(db, record.higher,
-					      rechigher.token);
+					      (char *) (rechigher.token));
 
 	/*
 	 * Now we add this record's offset to the head of the "free records"
