@@ -61,6 +61,48 @@ static void msg_parse__fromhdr(opts_t opts, msg_t msg, char *line,
 }
 
 
+/*
+ * Replace the existing msg->envsender, if there is one, with a malloc()ed
+ * string containing the email address part of the given "Return-Path:"
+ * header line. This works the same way as msg_parse__fromhdr() above.
+ */
+static void msg_parse__returnpathhdr(opts_t opts, msg_t msg, char *line,
+				     long size)
+{
+	long i, a, b;
+	char *ptr;
+
+	for (i = size - 1; i > 0 && line[i] != '@'; i--) {
+	}
+
+	if (line[i] != '@')
+		return;
+
+	for (a = i; a > 0 && line[a] != '<' && line[a] > 32; a--) {
+	}
+	if ((line[a] == '<') || (line[a] <= 32))
+		a++;
+
+	for (b = i; b < size && line[b] != '>' && line[b] > 32; b++) {
+	}
+	if ((line[b] == '>') || (line[b] <= 32))
+		b--;
+
+	if (b < (a + 2))
+		return;
+
+	ptr = calloc(1, 2 + b - a);
+	if (ptr == NULL)
+		return;
+	strncpy(ptr, line + a, 1 + b - a);
+
+	if (msg->envsender)
+		free(msg->envsender);
+
+	msg->envsender = ptr;
+}
+
+
 /*
  * Parse a single message header, updating internal state as appropriate.
  * Returns nonzero on error.
@@ -94,7 +136,7 @@ static int msg_parse__header(opts_t opts, msg_t msg)
 		 * Empty line - end of headers
 		 */
 
-		msg->_in_header = 0;
+		msg->_in_header--;
 
 	} else if (strncasecmp(msg->original + msg->_pos,
 			       "Content-Type:", 13) == 0) {
@@ -114,12 +156,25 @@ static int msg_parse__header(opts_t opts, msg_t msg)
 			msg->_pos++;
 		}
 		msg->_nottext = 1;
-		if (strncasecmp(msg->original + msg->_pos, "text/", 5)
-		    == 0)
+		if (strncasecmp(msg->original + msg->_pos, "text/", 5) ==
+		    0) {
 			msg->_nottext = 0;
-		if (strncasecmp(msg->original + msg->_pos, "image/", 6) ==
-		    0)
+		} else
+		    if (strncasecmp(msg->original + msg->_pos, "image/", 6)
+			== 0) {
 			msg->num_images++;
+		} else
+		    if (strncasecmp
+			(msg->original + msg->_pos, "message/", 8) == 0) {
+			/*
+			 * Handle message/ types by allowing a blank line
+			 * and then continuing to parse header lines, so
+			 * that attached messages with inline parts can be
+			 * processed properly.
+			 */
+			msg->_in_header = 2;
+			msg->_nottext = 0;
+		}
 		while (((msg->original[msg->_pos] & 0x60) != 'B')
 		       && (msg->_pos < end)
 		       && (strncasecmp(msg->original + msg->_pos,
@@ -197,6 +252,8 @@ static int msg_parse__header(opts_t opts, msg_t msg)
 				"Subject:", 8) == 0)
 		   || (strncasecmp(msg->original + msg->_pos,
 				   "From:", 5) == 0)
+		   || (strncasecmp(msg->original + msg->_pos,
+				   "Return-Path:", 12) == 0)
 		   || (strncasecmp(msg->original + msg->_pos,
 				   "Sender:", 7) == 0)
 		   || (strncasecmp(msg->original + msg->_pos,
@@ -224,6 +281,16 @@ static int msg_parse__header(opts_t opts, msg_t msg)
 			msg_parse__fromhdr(opts, msg,
 					   msg->original + msg->_pos,
 					   end + 1 - msg->_pos);
+		} else
+		    if (strncasecmp
+			(msg->original + msg->_pos, "Return-Path:", 12)
+			== 0) {
+			/*
+			 * Also store email address of envelope sender
+			 */
+			msg_parse__returnpathhdr(opts, msg,
+						 msg->original + msg->_pos,
+						 end + 1 - msg->_pos);
 		}
 
 		/*
@@ -256,6 +323,7 @@ static int msg_parse__content(opts_t opts, msg_t msg)
 	char digeststr[64];		 /* RATS: ignore (large enough) */
 	char *newptr;
 	char *data;
+	char *content;
 	long boundstart;		 /* start of message part being decoded */
 	long partsize;			 /* size of message part being decoded */
 	int i, n;
@@ -323,31 +391,30 @@ static int msg_parse__content(opts_t opts, msg_t msg)
 		break;
 	}
 
-	if (data != NULL) {
-		if (msg->_nottext) {
-			MD5Init(&md5c);
-			MD5Update(&md5c, (unsigned char *) data, partsize);
-			MD5Final(digest, &md5c);
-			strcpy(digeststr, " z");
-			for (n = 0; n < 16; n++) {
-				sprintf(digeststr + 2 + 2 * n, "%02X",
-					digest[n]);
-			}
-			strcat(digeststr, "z ");
-			if (msg_addcontent
-			    (opts, msg, digeststr, strlen(digeststr)))
-				return 1;
-		} else {
-			if (msg_addcontent(opts, msg, data, partsize))
-				return 1;
+	content = data;
+	if (content == NULL)
+		content = msg->original + boundstart;
+
+	if (msg->_nottext) {
+		MD5Init(&md5c);
+		MD5Update(&md5c, (unsigned char *) content, partsize);
+		MD5Final(digest, &md5c);
+		strcpy(digeststr, " z");
+		for (n = 0; n < 16; n++) {
+			sprintf(digeststr + 2 + 2 * n, "%02X", digest[n]);
 		}
-		free(data);
-	} else if (msg->_nottext != 1) {
-		if (msg_addcontent(opts, msg,
-				   msg->original + boundstart, partsize))
+		strcat(digeststr, "z ");
+		if (msg_addcontent
+		    (opts, msg, digeststr, strlen(digeststr)))
+			return 1;
+	} else {
+		if (msg_addcontent(opts, msg, content, partsize))
 			return 1;
 	}
 
+	if (data)
+		free(data);
+
 	if (msg->_in_header) {
 		msg->_nottext = 0;
 		msg->_encoding = 0;
@@ -740,7 +807,7 @@ msg_t msg_parse(opts_t opts)
 	 */
 	while (msg->_pos < msg->original_size) {
 
-		if (msg->_in_header) {
+		if (msg->_in_header > 0) {
 			if (msg_parse__header(opts, msg))
 				return msg;
 		} else {
