@@ -21,7 +21,7 @@ unsigned char *spam_checksum(char *key, int len)
 	unsigned char digest[64];	 /* RATS: ignore (size OK) */
 	char buf[16];			 /* RATS: ignore (size OK) */
 	char resultstr[128];		 /* RATS: ignore (size OK) */
-	char *ptr;
+	unsigned char *ptr;
 	int i;
 
 	MD5Init(&md5c);
@@ -34,7 +34,7 @@ unsigned char *spam_checksum(char *key, int len)
 		strcat(resultstr, buf);	    /* RATS: ignore (len OK) */
 	}
 
-	ptr = strdup(resultstr);
+	ptr = (unsigned char *) (strdup(resultstr));
 
 	if (ptr == NULL)
 		abort();
